# -*- coding: utf-8 -*-
from django.utils.translation import gettext_lazy as _
from rest_framework import exceptions, viewsets
from rest_framework.decorators import action
from rest_framework.response import Response
from thx.api.mixins import ThuxModelMixin

from ... import models
from . import serializers

STATUS_CODES = ((0, _('disable')), (1, _('enable')),)


# PROJECT TYPE
class ProjectTypeViewSet(ThuxModelMixin, viewsets.ModelViewSet):
    """
    ProjectTypeViewSet
    -----------

    :action list: Return the projecttype list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given projecttype.
        `Profiles`: `superuser`, `staff`

    :action light: Return the projecttype list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new projecttype instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update a projecttype instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch a projecttype instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Delete a projecttype instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.ProjectType.objects.all()
    serializer_class = serializers.ProjectTypeSerializer

    def get_queryset(self):
        user = self.request.user
        profile = self.kwargs.get('profile')
        qs = super().get_queryset()
        if user.is_superuser and profile == 'superuser':
            return qs
        if user.is_staff and profile == 'staff':
            return qs
        return qs.none()

    def check_permissions(self, request):
        super().check_permissions(request)
        user = self.request.user
        profile = self.kwargs.get('profile')
        if user.is_superuser and profile == 'superuser':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable', 'light',
                'status_choices'
            ]:
                return
        if user.is_staff and profile == 'staff':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable', 'light',
                'status_choices'
            ]:
                return
        raise exceptions.PermissionDenied()

    def get_serializer_class(self):
        action = self.action.replace('-', ' ').replace('_', ' ').title().replace(' ', '')
        serializer = getattr(serializers, f'ProjectType{action}Serializer', None)
        if serializer:
            return serializer
        return super().get_serializer_class()

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable a projecttype instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable a projecttype instance.
        `Profiles`: `superuser`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)


# PROJECT
class ProjectViewSet(ThuxModelMixin, viewsets.ModelViewSet):
    """
    ProjectViewSet
    -----------

    :action list: Return the project list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given project.
        `Profiles`: `superuser`, `staff`

    :action light: Return the project list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new project instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update a project instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch a project instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Delete a project instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.Project.objects.all()
    serializer_class = serializers.ProjectSerializer

    def get_queryset(self):
        user = self.request.user
        profile = self.kwargs.get('profile')
        qs = super().get_queryset()
        if user.is_superuser and profile == 'superuser':
            return qs
        if user.is_staff and profile == 'staff':
            return qs
        return qs.none()

    def check_permissions(self, request):
        super().check_permissions(request)
        user = self.request.user
        profile = self.kwargs.get('profile')
        if user.is_superuser and profile == 'superuser':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable', 'light',
                'status_choices'
            ]:
                return
        if user.is_staff and profile == 'staff':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable', 'light',
                'status_choices'
            ]:
                return
        raise exceptions.PermissionDenied()

    def get_serializer_class(self):
        action = self.action.replace('-', ' ').replace('_', ' ').title().replace(' ', '')
        serializer = getattr(serializers, f'Project{action}Serializer', None)
        if serializer:
            return serializer
        return super().get_serializer_class()

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable a project instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable a project instance.
        `Profiles`: `superuser`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)


# PROJECT NUMBER
class ProjectNumberViewSet(ThuxModelMixin, viewsets.ModelViewSet):
    """
    ProjectNumberViewSet
    -----------

    :action list: Return the projectnumber list.
        `Profiles`: `superuser`, `staff`

    :action retrieve: Return the given projectnumber.
        `Profiles`: `superuser`, `staff`

    :action light: Return the projectnumber list using light serialization.
        `Profiles`: `superuser`, `staff`

    :action create: Create a new projectnumber instance.
        `Profiles`: `superuser`, `staff`

    :action update: Update a projectnumber instance.
        `Profiles`: `superuser`, `staff`

    :action partial_update: Patch a projectnumber instance.
        `Profiles`: `superuser`, `staff`

    :action destroy: Delete a projectnumber instance.
        `Profiles`: `superuser`, `staff`
    """
    queryset = models.ProjectNumber.objects.all()
    serializer_class = serializers.ProjectNumberSerializer

    def get_queryset(self):
        user = self.request.user
        profile = self.kwargs.get('profile')
        qs = super().get_queryset()
        if user.is_superuser and profile == 'superuser':
            return qs
        if user.is_staff and profile == 'staff':
            return qs
        return qs.none()

    def check_permissions(self, request):
        super().check_permissions(request)
        user = self.request.user
        profile = self.kwargs.get('profile')
        if user.is_superuser and profile == 'superuser':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable', 'light',
                'status_choices'
            ]:
                return
        if user.is_staff and profile == 'staff':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable', 'light',
                'status_choices'
            ]:
                return
        raise exceptions.PermissionDenied()

    def get_serializer_class(self):
        action = self.action.replace('-', ' ').replace('_', ' ').title().replace(' ', '')
        serializer = getattr(serializers, f'ProjectNumber{action}Serializer', None)
        if serializer:
            return serializer
        return super().get_serializer_class()

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        """
        Enable a projectnumber instance.
        `Profiles`: `superuser`, `staff`
        """
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        """
        Disable a projectnumber instance.
        `Profiles`: `superuser`
        """
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        """
        Return the status choices options.
        `Profiles`: `superuser`, `staff`
        """
        return Response(STATUS_CODES)
